<?php
require_once DIR_SYSTEM . 'library/VerifoneEcomApi/autoload.php';

use VerifoneEcomApi\ApiWrapper\Settings;
use Verifone\Verifone_Helpers;

class ControllerExtensionPaymentVerifoneHosted extends Controller
{
    public $settings;
    private $error = [];
    public $verifoneKeys = [
        '_status',
        '_test',
        '_total',
        '_order_status_id',
        '_pending_status_id',
        '_refunded_status_id',
        '_failed_status_id',
        '_geo_zone_id',
        '_cart_payment_type',
        '_region',
        '_prod_api_user',
        '_prod_api_key',
        '_prod_entity',
        '_prod_contract',
        '_prod_3ds_contract',
        '_card_status',
        '_3ds',
        '_cvv',
        '_curl',
        '_overlay',
        '_fraud',
        '_fraud_contract_id',
        '_applepay_status',
        '_applepay_contract',
        '_applepay_sca',
        '_googlepay_status',
        '_googlepay_contract',
        '_googlepay_sca',
        '_vipps_status',
        '_vipps_contract',
        '_vipps_3ds_contract',
        '_vipps_sca',
        '_mobile_pay_status',
        '_mobile_pay_contract',
        '_mobile_pay_3ds_contract',
        '_mobile_pay_sca',
        '_klarna_status',
        '_swish_status',
        '_gpp2_status',
        '_gpp2_contract',
        '_abs_status',
        '_abs_installments',
        '_paypal_status',
        '_paypal_contract',
        '_transaction_type',
        '_gift_card_status',
        '_gift_card_contract',
        '_plcc_status',
        '_plcc_contract',
    ];
    /**
     * name of the payment method`
     */
    const NAME = 'payment_verifone_hosted';

    /**
     * returns the settings view
     */
    public function index()
    {
        $this->load->language('extension/payment/verifone_hosted');
        $this->document->setTitle($this->language->get('heading_title'));
        $this->load->model('setting/setting');
        $this->load->model('localisation/order_status');
        $this->load->model('localisation/geo_zone');

        $post_data = $this->request->post;

        if (($this->request->server['REQUEST_METHOD'] == 'POST') && $this->validate()) {
            $this->model_setting_setting->editSetting('payment_verifone_hosted', $post_data);
            $this->session->data['success'] = $this->language->get('text_success');
            $this->response->redirect($this->url->link('marketplace/extension', 'user_token=' . $this->session->data['user_token'] . '&type=payment', true));
        }

        $data['error_warning'] = $this->error['warning'] ?? '';
        $data['errors'] = $this->error['errors'] ?? [];

        $data['breadcrumbs'] = [
            [
                'text' => $this->language->get('text_home'),
                'href' => $this->url->link('common/dashboard', 'user_token=' . $this->session->data['user_token'], true)
            ],
            [
                'text' => $this->language->get('text_extension'),
                'href' => $this->url->link('marketplace/extension', 'user_token=' . $this->session->data['user_token'] . '&type=payment', true)
            ],
            [
                'text' => $this->language->get('heading_title'),
                'href' => $this->url->link('extension/payment/verifone', 'user_token=' . $this->session->data['user_token'], true)
            ]
        ];

        $data['order_statuses'] = $this->model_localisation_order_status->getOrderStatuses();
        $data['action'] = $this->url->link('extension/payment/verifone_hosted', 'user_token=' . $this->session->data['user_token'], true);
        $data['cancel'] = $this->url->link('marketplace/extension', 'user_token=' . $this->session->data['user_token'] . '&type=payment', true);
        $data['geo_zones'] = $this->model_localisation_geo_zone->getGeoZones();

        foreach ($this->verifoneKeys as $key) {
            $data[self::NAME . $key] = $this->config->get(self::NAME . $key) ?? '';
        }

        $data['header'] = $this->load->controller('common/header');
        $data['column_left'] = $this->load->controller('common/column_left');
        $data['footer'] = $this->load->controller('common/footer');
        $this->settings = new Settings();
        $data['missing_order'] = $this->settings::MISSING_ORDER;
        $data['webhook_url'] = $this->get_webhook_response_url();

        $this->response->setOutput($this->load->view('extension/payment/verifone_hosted', $data));
    }

    /**
     * validates the Save post request
     * @return bool
     */
    protected function validate()
    {
        if (!$this->user->hasPermission('modify', 'extension/payment/verifone_hosted')) {
            $this->error['warning'] = $this->language->get('error_permission');
        }
        $except = [self::NAME . '_test_3ds_contract', self::NAME . '_prod_3ds_contract'];
        $post_data = $this->request->post;
        foreach ($post_data as $key => $item) {

            // all fields from test/prod tabs are required
            if (strpos($key, 'payment_verifone_hosted_prod_') !== false ||
                strpos($key, 'payment_verifone_hosted_test_') !== false) {
                if (!$item) {
                    if (!in_array($key, $except)) {
                        $lang_key = str_replace('payment_verifone_hosted_', 'entry_', $key);
                        $this->error['errors'][] = $this->language->get($lang_key) . $this->language->get('error_required_field');
                    }
                }
            }
        }

        return !$this->error;
    }

    /**
     * install the payment module
     */
    public function install()
    {
        $this->load->model('extension/payment/verifone_hosted');

        $this->model_extension_payment_verifone_hosted->install();
    }

    /**
     * uninstall the payment module
     */
    public function uninstall()
    {
        $this->load->model('extension/payment/verifone_hosted');

        $this->model_extension_payment_verifone_hosted->uninstall();
    }

    /**
     * refunds an order
     */
    public function refund()
    {
        //default response
        $response = ['success' => false, 'message' => 'Missing data'];

        if (isset($this->request->post['order_id']) && !empty($this->request->post['order_id']) &&
            isset($this->request->post['amount']) && !empty($this->request->post['amount'])) {
            $this->load->model('extension/payment/verifone_hosted');
            $response = $this->model_extension_payment_verifone_hosted->refund(
                $this->request->post['order_id'],
                $this->request->post['amount'],
                $this->request->post['comment']
            );
        }
        $this->response->addHeader('Content-Type: application/json');
        $this->response->setOutput(json_encode($response));
    }


    public function capture()
    {
        //default response
        $response = ['success' => false, 'message' => 'Missing data'];

        if (!empty($this->request->post['order_id']) && !empty($this->request->post['capture_amount'])) {
            $this->load->model('extension/payment/verifone_hosted');
            $response = $this->model_extension_payment_verifone_hosted->capture(
                $this->request->post['order_id'],
                $this->request->post['capture_amount']
            );
        }
        $this->response->addHeader('Content-Type: application/json');
        $this->response->setOutput(json_encode($response));
    }

    /**
     * @return mixed
     * @throws Exception
     */
    public function order()
    {
        $this->load->language('extension/payment/verifone_hosted');
        $this->load->model('extension/payment/verifone_hosted');
        $this->load->model('sale/order');

        $order = $this->model_sale_order->getOrder($this->request->get['order_id']);
        if ($this->config->get('payment_verifone_hosted_status') && $order['payment_code'] === 'verifone_hosted') {
            $transaction = $this->model_extension_payment_verifone_hosted->getTransactionByOrderId($order['order_id']);

            if (isset($transaction)) {
                $available_amount = Verifone_Helpers::calculateAmount($transaction['amount_captured']);

                if ($transaction['refunds']) {
                    foreach ($transaction['refunds'] as $refund) {
                        if ($refund['verifone_refund_status'] === 'REFUNDED') {
                            $amount_refunded = (int)Verifone_Helpers::calculateAmount($refund['amount_refunded']);
                            $available_amount = $available_amount - $amount_refunded;
                        }
                    }
                }
                $verifoneTransaction = $this->model_extension_payment_verifone_hosted->getVerifoneTransaction($transaction);
                // in case was changed before...
                $transaction['available_amount'] = $available_amount / 100;
                $transaction['amount_refunded'] = number_format(((Verifone_Helpers::calculateAmount($transaction['amount_captured']) - $available_amount) / 100), 2);
                $transaction['amount_captured'] = number_format($transaction['amount_captured'], 2);
                $data['text_refund'] = $this->language->get('text_refund');
                $data['text_total_amount_refund'] = $this->language->get('text_total_amount_refund');
                $data['text_refund_final'] = $this->language->get('text_refund_final');
                $data['action'] = $this->url->link('extension/payment/verifone_hosted/refund', 'user_token=' . $this->session->data['user_token'], true);
                $data['actionCapture'] = $this->url->link('extension/payment/verifone_hosted/capture', 'user_token=' . $this->session->data['user_token'], true);
                $data['actionVoid'] = $this->url->link('extension/payment/verifone_hosted/void', 'user_token=' . $this->session->data['user_token'], true);
                $data['actionVoidCapture'] = $this->url->link('extension/payment/verifone_hosted/voidCapture', 'user_token=' . $this->session->data['user_token'], true);
                $data['order'] = $order;

                if ($verifoneTransaction) {
                    $transaction['verifone_order_status'] = $verifoneTransaction['status'];
                    $data['can_refund'] = strpos($verifoneTransaction['status'], 'SETTLED') !== false && $available_amount;
                    $data['can_void_capture'] = in_array($verifoneTransaction['status'], ['AUTHORISATION AUTHORISED', 'PREAUTH AUTHORISED', 'CAPTURE FAILED']);
                    $data['can_void'] = $verifoneTransaction['status'] === 'CAPTURE AUTHORISED';
                } else {
                    $data['can_refund'] = false;
                    $data['can_void_capture'] = false;
                    $data['can_void'] = false;
                }

                $data['user_token'] = $this->session->data['user_token'];
                $data['transaction'] = $transaction;

            }

            $data['order_total'] = $this->currency->format($order['total'], $order['currency_code'], '', false);
            return $this->load->view('extension/payment/verifone_order', $data);
        }
    }

    /**
     * @return string
     */
    public function get_webhook_response_url()
    {
        return str_replace('admin/index.php', 'index.php', $this->url->link('extension/module/verifone_hosted/process_webhook', false));
    }

}
