<?php

require_once DIR_SYSTEM . 'library/VerifoneEcomApi/autoload.php';

use Verifone\Verifone_Helpers;
use VerifoneEcomApi\ApiWrapper\Authentication\BasicAuth;
use VerifoneEcomApi\ApiWrapper\Http\ApiClient\Client;
use VerifoneEcomApi\ApiWrapper\Http\SimpleCurl;
use VerifoneEcomApi\ApiWrapper\Regions\Emea;
use VerifoneEcomApi\ApiWrapper\Regions\Na;
use VerifoneEcomApi\ApiWrapper\Regions\NzAu;
use VerifoneEcomApi\ApiWrapper\Settings;

class ModelExtensionPaymentVerifoneHosted extends Model
{
    private $mapper;
    private $simpleCurl;
    private $auth;
    private $settings;

    /**
     * install the module & creates the table
     */
    public function install()
    {
        $this->db->query("
            CREATE TABLE IF NOT EXISTS `" . DB_PREFIX . "verifone_hosted` (
            `id` INT(11) NOT NULL AUTO_INCREMENT ,
            `order_id` INT(11) NOT NULL ,
            `amount` DECIMAL(10,2) NOT NULL ,
            `amount_captured` DECIMAL(10,2) NULL ,
            `currency` VARCHAR(3) NULL ,
            `transaction_id` VARCHAR(50) NOT NULL ,
            `verifone_order_status` VARCHAR(50) NOT NULL ,
            `created_at` TIMESTAMP NOT NULL DEFAULT CURRENT_TIMESTAMP ,
            PRIMARY KEY (`id`)
            ) ENGINE = InnoDB;");

        $this->db->query("
            CREATE TABLE IF NOT EXISTS `" . DB_PREFIX . "verifone_hosted_refunds` (
            `id` INT(11) NOT NULL AUTO_INCREMENT ,
            `order_id` INT(11) NOT NULL ,
            `amount_refunded` DECIMAL(10,2) NOT NULL ,
            `verifone_refund_status` VARCHAR(50) NOT NULL ,
            `comment` TEXT NULL ,
            `response` TEXT NULL ,
            `transaction_id` TEXT NULL ,
            `created_at` TIMESTAMP NOT NULL DEFAULT CURRENT_TIMESTAMP ,
            PRIMARY KEY (`id`)
            ) ENGINE = InnoDB;");
        $this->addColumn('verifone_hosted', 'amount_captured');
    }

    /**
     * @param $table
     * @param $column
     * @return void
     */
    private function addColumn($table, $column)
    {
        $amountCapturedColumn = "IF NOT EXISTS( SELECT NULL FROM INFORMATION_SCHEMA.COLUMNS WHERE 
        table_name = '" . DB_PREFIX . $table . "' AND table_schema = '" . DB_DATABASE . "' 
        AND column_name = '" . $column . "')  
            THEN
                ALTER TABLE `" . DB_PREFIX . $table . "` ADD `" . $column . "` DECIMAL(10,2) NULL;
            END IF;";
        $this->db->query($amountCapturedColumn);
    }

    /**
     * uninstall and drop the table
     */
    public function uninstall()
    {
    }

    /**
     *
     */
    public function setUpVerifoneSettings()
    {
        $this->settings = new Settings();
        $this->settings->setTest('yes' == $this->config->get('payment_verifone_hosted_test'))
            ->setRegion($this->config->get('payment_verifone_hosted_region'))
            ->setApiUserId($this->config->get('payment_verifone_hosted_prod_api_user'))
            ->setApiKey($this->config->get('payment_verifone_hosted_prod_api_key'))
            ->setCardEnable('yes' == $this->config->get('payment_verifone_hosted_card_status'))
            ->set3dsEnable('yes' == $this->config->get('payment_verifone_hosted_3ds'))
            ->set3dsContractId($this->config->get('payment_verifone_hosted_prod_3ds_contract'))
            ->setPaymentContractId($this->config->get('payment_verifone_hosted_prod_contract'))
            ->setEntityId($this->config->get('payment_verifone_hosted_prod_entity'))
            ->setCurlVerifySsl('1' == $this->config->get('payment_verifone_hosted_curl'))
            ->setFraudCheckEnabled('yes' == $this->config->get('payment_verifone_hosted_fraud'))
            ->setFraudCheckContractId($this->config->get('payment_verifone_hosted_fraud_contract_id'))
            ->setIsApplePayEnabled('yes' == $this->config->get('payment_verifone_hosted_applepay_status'))
            ->setApplePayContractId($this->config->get('payment_verifone_hosted_applepay_contract'))
            ->setApplePayScaComplianceLevel($this->config->get('payment_verifone_hosted_applepay_sca'))
            ->setIsGooglePayEnabled('yes' == $this->config->get('payment_verifone_hosted_googlepay_status'))
            ->setGooglePayContractId($this->config->get('payment_verifone_hosted_googlepay_contract'))
            ->setGooglePayScaComplianceLevel($this->config->get('payment_verifone_hosted_googlepay_sca'))
            ->setIsPayPalEnabled('yes' == $this->config->get('payment_verifone_hosted_paypal_status'))
            ->setPayPalContractId($this->config->get('payment_verifone_hosted_paypal_contract'))
            ->setIsVippsEnabled('yes' == $this->config->get('payment_verifone_hosted_vipps_status'))
            ->setKlarnaEnabled('yes' == $this->config->get('payment_verifone_hosted_klarna_status'))
            ->setVippsContractId($this->config->get('payment_verifone_hosted_vipps_contract'))
            ->setVippsScaComplianceLevel($this->config->get('payment_verifone_hosted_vipps_sca'))
            ->setVipps3DSContractId($this->config->get('payment_verifone_hosted_vipps_3ds_contract'))
            ->setGiftCardEnabled($this->config->get('payment_verifone_hosted_gift_card_status'))
            ->setGiftCardContractId($this->config->get('payment_verifone_hosted_gift_card_contract'))
            ->setIsSwishEnabled($this->config->get('payment_verifone_hosted_swish_status'))
            ->setIsGpp2Enabled($this->config->get('payment_verifone_hosted_gpp2_status'))
            ->setGpp2PayContractId($this->config->get('payment_verifone_hosted_gpp2_contract'))
            ->setIsAbsEnabled($this->config->get('payment_verifone_hosted_abs_status'))
            ->setAbsInstallments($this->config->get('payment_verifone_hosted_abs_installments'))
            ->setTransactionType($this->config->get('payment_verifone_hosted_transaction_type'))
            ->setPlccEnabled($this->config->get('payment_verifone_hosted_plcc_status'))
            ->setPlccContractId($this->config->get('payment_verifone_hosted_plcc_contract'));
    }

    /**
     *
     */
    public function getRegionMapper()
    {
        // mini factory :D
        switch ($this->config->get('payment_verifone_hosted_region')) {
            case 'NZAU':
                $this->mapper = new Nzau($this->settings);
                break;
            case 'NA':
                $this->mapper = new Na($this->settings);
                break;
            case 'EMEA':
                $this->mapper = new Emea($this->settings);
                break;
            default:
                // will happen only on the first installation
                $this->mapper = new Nzau($this->settings);
        }
    }

    /**
     * @param $order_id
     * @return mixed
     */
    public function getTransactionByOrderId($order_id)
    {
        $query = $this->db->query("SELECT * FROM `" . DB_PREFIX . "verifone_hosted` WHERE `order_id` = '" . (int)$order_id . "' ORDER BY `created_at` DESC LIMIT 1");

        if ($query->num_rows) {
            $transaction = $query->row;
            $q = $this->db->query("SELECT * FROM `" . DB_PREFIX . "verifone_hosted_refunds` WHERE `order_id` = '" . $order_id . "'");
            $transaction['refunds'] = ($q->num_rows) ? $q->rows : [];

            return $transaction;
        }

        return null;
    }

    public function createRefund($amount, $comment)
    {
        return [
            'amount' => (int)Verifone_Helpers::calculateAmount($amount),
            'reason' => (string)$comment,
        ];
    }

    /**
     * @param        $order_id
     * @param        $amount
     * @param string $comment
     * @return array
     * @throws Exception
     */
    public function refund($order_id, $amount, string $comment = '')
    {
        $this->load->model('sale/order');
        $order = $this->model_sale_order->getOrder($order_id);
        $transaction = $this->getTransactionByOrderId($order_id);
        $available_amount = $transaction['amount'];
        $amount = number_format((float)$amount, 2, '.', '');
        if ($transaction['refunds']) {
            foreach ($transaction['refunds'] as $refund) {
                if ($refund['verifone_refund_status'] === 'REFUNDED') {
                    $available_amount = $available_amount - $refund['amount_refunded'];
                }
            }
        }
        $available_amount = round($available_amount, 2);
        if ($amount > $available_amount) {
            return [
                'success' => false,
                'message' => 'Amount that can be refunded must be between 0.1 '
                    . $order['currency_code'] . ' and ' . $available_amount . ' ' . $order['currency_code'] . '!'
            ];
        }

        $client = $this->getClient();

        $params = $this->createRefund($amount, $comment);

        try {
            $result = $client->postRefund($params, $transaction["transaction_id"]);
        } catch (Exception $e) {

            return [
                'success' => false,
                'message' => $e->getMessage(),
                'full_refund' => false,
                'row' => '<tr><td colspan="5">Something went wrong. The request was not processed!</td></tr>'
            ];
        }

        //REFUNDED - ok, other statuses are not ok
        if (!isset($result['id']) || !isset($result['status']) || 'REFUNDED' !== $result['status']) { // with error
            $success = false;
            $response = 'Something went wrong, please try again later or contact us';
            return [
                'success' => $success,
                'message' => $response,
                'full_refund' => false,
                'row' => '<tr><td colspan="5">Something went wrong. The request was not processed!</td></tr>'
            ];
        } else { // success response, but not necessary a success refund
            $status = $result['status'];
            $success = true;
            $response = 'Successfully refunded';
        }
        $full_refund = false;
        // if is a full refund, we update the order status
        if ($amount == $available_amount) {
            $this->setOrderAsRefunded($order_id);
            $full_refund = true;
        }

        return [
            'success' => $success,
            'message' => $response,
            'full_refund' => $full_refund,
            'row' => $this->createRefundTransaction($status, $response, $order_id, $amount, $comment, $result['id'])
        ];
    }

    /**
     * @param $status
     * @param $response
     * @param $order_id
     * @param $amount
     * @param $comment
     * @param $transaction_id
     * @return string
     */
    private function createRefundTransaction($status, $response, $order_id, $amount, $comment, $transaction_id)
    {

        $insert = $this->db->query("
                INSERT INTO `" . DB_PREFIX . "verifone_hosted_refunds` SET
                `order_id` = '" . $order_id . "',
                `amount_refunded` = '" . $amount . "',
                `verifone_refund_status` = '" . $this->db->escape($status) . "',
                `comment` = '" . $this->db->escape($comment) . "',
                `response` = '" . $this->db->escape($response) . "',
                `transaction_id` = '" . $this->db->escape($transaction_id) . "'
                ");
        if (!$insert) {
            return '<tr><td colspan="5">Something went wrong. The request was not processed!</td></tr>';
        }
        $class = $status === 'REFUNDED' ? 'text-success' : 'text-warning';
        $formatted_status = str_replace('_', ' ', $status);

        return '<tr>
                    <td>' . $amount . '</td>
                    <td><strong class="' . $class . '">' . $formatted_status . '</strong></td>
                    <td>' . $response . '</td>
                    <td>' . $comment . '</td>
                    <td>' . date('Y-m-d H:i:s', time()) . '</td>
               </tr>';
    }

    /**
     * @param $order_id
     */
    private function setOrderAsRefunded($order_id)
    {

        $this->db->query("UPDATE `" . DB_PREFIX . "order` SET 
                `order_status_id` =  '" . (int)$this->config->get('payment_verifone_hosted_refunded_status_id') . "'
                WHERE `order_id` = '" . $order_id . "'");

        $this->db->query("INSERT INTO `" . DB_PREFIX . "order_history` SET 
                `order_id` =  '" . $order_id . "',
                `order_status_id` =  '" . (int)$this->config->get('payment_verifone_hosted_refunded_status_id') . "',
                `date_added` =  '" . date('Y-m-d H:i:s', time()) . "',
            `comment` =  'Full amount refunded'");

    }


    public function capture($order_id, $amount): array
    {
        $this->load->model('sale/order');
        $order = $this->model_sale_order->getOrder($order_id);

        $amount = number_format((float)$amount, 2, '.', '');
        $transaction = $this->getTransactionByOrderId($order_id);
        if ($amount > $transaction['amount']) {
            return [
                'success' => false,
                'message' => 'Amount that can be capture must be between 0.1 '
                    . $order['currency_code'] . ' and ' . $transaction['amount'] . ' ' . $order['currency_code'] . '!'
            ];
        }

        $client = $this->getClient();
        $calculatedAmount = (string)Verifone_Helpers::calculateAmount($amount);
        $params = ['amount' => (int)$calculatedAmount];

        try {
            $result = $client->postCapture($params, $transaction["transaction_id"]);
        } catch (Exception $e) {

            return [
                'success' => false,
                'message' => $e->getMessage(),
            ];
        }

        if (!isset($result['id']) || !isset($result['status']) || !in_array($result['status'], ['SETTLED', 'AUTHORIZED','SETTLEMENT_COMPLETED'])) { // with error
            return [
                'result' => $result,
                'params' => $params,
                'success' => false,
                'message' => 'Something went wrong, please try again later or contact us',
            ];
        }


        // success response - amount captured, but not settled yet
        $response = $amount . ' ' . $order['currency_code'] . ' successfully captured. Refunds are available after the transaction is SETTLED, depending on your acquirer!';
        $comment = $amount . ' ' . $order['currency_code'] . ' successfully captured';
        // Update the DB with the new statuses
        $order_status_id = $this->config->get('payment_verifone_hosted_order_status_id');
        $this->db->query("UPDATE `" . DB_PREFIX . "order` SET order_status_id = '" . (int)$order_status_id . "', date_modified = NOW() WHERE order_id = '" . (int)$order_id . "'");
        $this->db->query("UPDATE `" . DB_PREFIX . "verifone_hosted` SET amount_captured = '" . $amount . "', verifone_order_status = 'TRANSACTION_SUCCESS' WHERE order_id = '" . (int)$order_id . "'");
        $this->db->query("INSERT INTO `" . DB_PREFIX . "order_history` SET order_id = '" . (int)$order_id . "', order_status_id = '" . (int)$order_status_id . "', notify = '0', comment = '" . $comment . "', date_added = NOW()");

        return [
            'success' => true,
            'message' => $response,
            'result' => $result
        ];
    }

    /**
     * @throws \VerifoneEcomApi\ApiWrapper\Http\HttpException
     */
    public function getVerifoneTransaction($transaction)
    {
        $verifoneTransaction =  $this->getClient()->getTransaction($transaction['transaction_id']);
        if (!$verifoneTransaction || !isset($verifoneTransaction['status'])) {
            return null;
        }

        if ($verifoneTransaction['status'] !== $transaction['verifone_order_status']) {
            $this->db->query("UPDATE `" . DB_PREFIX . "verifone_hosted` SET verifone_order_status = '" . $verifoneTransaction['status'] . "'WHERE transaction_id = '" . $transaction['transaction_id'] . "'");
        }

        return $verifoneTransaction;
    }

    public function void()
    {
        //void transaction
    }

    public function voidCapture()
    {
        //void captured transaction
    }

    /**
     * @return Client
     */
    private function getClient(): Client
    {
        $this->simpleCurl = new SimpleCurl();
        $this->setUpVerifoneSettings();
        $this->getRegionMapper();
        $this->auth = new BasicAuth($this->settings);
        return new Client($this->mapper, $this->settings, $this->auth, $this->simpleCurl);
    }
}
