<?php

require_once DIR_SYSTEM . 'library/VerifoneEcomApi/autoload.php';

use Verifone\Mobile_Detect;
use Verifone\Verifone_Helpers;
use VerifoneEcomApi\ApiWrapper\Authentication\BasicAuth;
use VerifoneEcomApi\ApiWrapper\Http\ApiClient\Client;
use VerifoneEcomApi\ApiWrapper\Http\SimpleCurl;
use VerifoneEcomApi\ApiWrapper\Regions\Emea;
use VerifoneEcomApi\ApiWrapper\Regions\Na;
use VerifoneEcomApi\ApiWrapper\Regions\NzAu;
use VerifoneEcomApi\ApiWrapper\Settings;

class ModelExtensionPaymentVerifoneHosted extends Model
{
    public $settings;
    private $mapper;
    private $simpleCurl;
    private $client;
    private $auth;

    public function __construct($registry)
    {
        parent::__construct($registry);
        $this->load->model('checkout/order');
        $this->load->model('localisation/order_status');
        $this->simpleCurl = new SimpleCurl();
        $this->settings = new Settings();
        $this->settings->setTest('yes' == $this->config->get('payment_verifone_hosted_test'))
            ->setRegion($this->config->get('payment_verifone_hosted_region'))
            ->setThemeId($this->config->get('payment_verifone_hosted_theme_id'))
            ->setApiUserId($this->config->get('payment_verifone_hosted_prod_api_user'))
            ->setApiKey($this->config->get('payment_verifone_hosted_prod_api_key'))
            ->setCardEnable('yes' == $this->config->get('payment_verifone_hosted_card_status'))
            ->set3dsEnable('yes' == $this->config->get('payment_verifone_hosted_3ds'))
            ->set3dsContractId($this->config->get('payment_verifone_hosted_prod_3ds_contract'))
            ->setPaymentContractId($this->config->get('payment_verifone_hosted_prod_contract'))
            ->setEntityId($this->config->get('payment_verifone_hosted_prod_entity'))
            ->setCurlVerifySsl('1' == $this->config->get('payment_verifone_hosted_curl'))
            ->setFraudCheckEnabled('yes' == $this->config->get('payment_verifone_hosted_fraud'))
            ->setFraudCheckContractId($this->config->get('payment_verifone_hosted_fraud_contract_id'))
            ->setIsApplePayEnabled('yes' == $this->config->get('payment_verifone_hosted_applepay_status'))
            ->setApplePayContractId($this->config->get('payment_verifone_hosted_applepay_contract'))
            ->setApplePayScaComplianceLevel($this->config->get('payment_verifone_hosted_applepay_sca'))
            ->setIsGooglePayEnabled('yes' == $this->config->get('payment_verifone_hosted_googlepay_status'))
            ->setGooglePayContractId($this->config->get('payment_verifone_hosted_googlepay_contract'))
            ->setGooglePayScaComplianceLevel($this->config->get('payment_verifone_hosted_googlepay_sca'))
            ->setIsPayPalEnabled('yes' == $this->config->get('payment_verifone_hosted_paypal_status'))
            ->setPayPalContractId($this->config->get('payment_verifone_hosted_paypal_contract'))
            ->setKlarnaEnabled('yes' == $this->config->get('payment_verifone_hosted_klarna_status'))
            ->setIsVippsEnabled('yes' == $this->config->get('payment_verifone_hosted_vipps_status'))
            ->setVippsContractId($this->config->get('payment_verifone_hosted_vipps_contract'))
            ->setVippsScaComplianceLevel($this->config->get('payment_verifone_hosted_vipps_sca'))
            ->setVipps3DSContractId($this->config->get('payment_verifone_hosted_vipps_3ds_contract'))
            ->setIsMobilePayEnabled('yes' == $this->config->get('payment_verifone_hosted_mobile_pay_status'))
            ->setMobilePayContractId($this->config->get('payment_verifone_hosted_mobile_pay_contract'))
            ->setMobilePayScaComplianceLevel($this->config->get('payment_verifone_hosted_mobile_pay_sca'))
            ->setMobilePay3DSContractId($this->config->get('payment_verifone_hosted_mobile_pay_3ds_contract'))
            ->setIsAbsEnabled($this->config->get('payment_verifone_hosted_abs_status'))
            ->setAbsInstallments($this->config->get('payment_verifone_hosted_abs_installments'))
            ->setGiftCardEnabled('yes' == $this->config->get('payment_verifone_hosted_gift_card_status'))
            ->setGiftCardContractId($this->config->get('payment_verifone_hosted_gift_card_contract'))
            ->setIsSwishEnabled($this->config->get('payment_verifone_hosted_swish_status'))
            ->setTransactionType($this->config->get('payment_verifone_hosted_transaction_type'))
            ->setIsGpp2Enabled($this->config->get('payment_verifone_hosted_gpp2_status'))
            ->setGpp2PayContractId($this->config->get('payment_verifone_hosted_gpp2_contract'))
            ->setPlccEnabled($this->config->get('payment_verifone_hosted_plcc_status'))
            ->setPlccContractId($this->config->get('payment_verifone_hosted_plcc_contract'))
            ->setPaymentType(
                Settings::PAYMENT_TYPE_IFRAME === $this->config->get('payment_verifone_hosted_cart_payment_type')
                    ? Settings::PAYMENT_TYPE_IFRAME
                    : Settings::PAYMENT_TYPE_HPP
            );
        $this->getRegionMapper();
        $this->auth = new BasicAuth($this->settings);
        $this->client = new Client(
            $this->mapper,
            $this->settings,
            $this->auth,
            $this->simpleCurl
        );
    }

    /**
     * @return Client
     */
    public function getClient()
    {
        return $this->client;
    }

    /**
     *
     */
    public function getRegionMapper()
    {
        // mini factory :D
        switch ($this->config->get('payment_verifone_hosted_region')) {
            case 'NZAU':
                $this->mapper = new Nzau($this->settings);
                break;
            case 'NA':
                $this->mapper = new Na($this->settings);
                break;
            case 'EMEA':
                $this->mapper = new Emea($this->settings);
                break;
            default:
                // will happen only on the first installation
                $this->mapper = new Nzau($this->settings);
        }
    }

    /**
     * @param $params
     * @return bool
     * @throws Exception
     */
    public function updateOrder($params)
    {

        try {
            $result = $this->client->getCheckout($params['checkout_id']);
        } catch (Exception $e) {
            throw new Exception($e->getMessage());
        }
        if ((isset($result['code']) && $result['code'] && $result['message']) || !isset($result['events'])) {

            throw new Exception($result['message'] ?? 'There was an error in the application!');
        }
        $success = false;
        $verifoneHelpers = new Verifone_Helpers($this->registry);
        foreach ($result['events'] as $event) {
            if ('TRANSACTION_SUCCESS' == $event['type']) {
                $orderStatus = $this->settings->getCapture() ? $this->config->get('payment_verifone_hosted_order_status_id') : $this->config->get('payment_verifone_hosted_pending_status_id');
                $verifoneHelpers->addOrderHistory(
                    $this->model_checkout_order,
                    $result['merchant_reference'],
                    (int)$orderStatus,
                    $this->language->get('entry_transaction_id') . '<strong style="color: #0D4AA2">' . $result['transaction_id'] . '</strong>'
                );
                $this->addTransaction($result, $event['type']);
                $success = true;

            }
            if (in_array($event['type'], ['TRANSACTION_FAILED', 'TRANSACTION_DECLINED'])) {
                $verifoneHelpers->addOrderHistory(
                    $this->model_checkout_order,
                    $result['merchant_reference'],
                    (int)$this->config->get('payment_verifone_hosted_failed_status_id'),
                    $this->language->get('entry_transaction_id') . '<strong style="color: #dc3050">' . $result['transaction_id'] . '</strong>'
                );
                $this->addTransaction($result, $event['type']);
            }
        }
        return $success;
    }

    /**
     * once the payment is made, we save the transaction for future refunds
     * @param $params
     * @param $status
     */
    public function addTransaction($params, $status)
    {
        $total = $this->calculateAmount($params['amount'], $params['currency_code'], false);
        $capturedAmount = $this->settings->getCapture() ? $total : 0;
        $this->db->query("
                INSERT INTO `" . DB_PREFIX . "verifone_hosted` SET
                `order_id` = '" . (int)$params['merchant_reference'] . "',
                `transaction_id` = '" . $this->db->escape($params['transaction_id']) . "',
                `amount` = '" . $total . "',
                `amount_captured` = '" . $capturedAmount . "',
                `currency` = '" . $this->db->escape($params['currency_code']) . "',
                `verifone_order_status` = '" . $this->db->escape($status) . "'
                ");
    }


    /**
     * @throws Exception
     */
    public function create($order_id)
    {
        $order_info = $this->model_checkout_order->getOrder($order_id);
        $postCustomer = $this->buildCustomer($order_info);
        try {
            $customer = $this->client->postCustomer($postCustomer);
        } catch (Exception $e) {
            throw new Exception($e->getMessage());
        }
        if (!isset($customer['id'])) {
            throw new Exception('Failed to create customer ' . ($customer['message'] ?? '') . '.');
        }
        $customer_id = $customer['id'];

        $total = $this->currency->format($order_info['total'], $order_info['currency_code'], $order_info['currency_value'], false);
        $order_params = $this->buildOrder($order_info, $customer_id, $total);
        try {
            $result = $this->client->postCheckout($order_params);
        } catch (Exception $e) {
            throw new Exception($e->getMessage());
        }
        $message = 'Something went wrong. Checkout was not created!';
        if (isset($result['code']) || !isset($result['url'])) { //error
            if (isset($result['details']) && !empty($result['details']['message'])) {
                $message = $result['details']['message'];
            }
            $return = [
                'success' => false,
                'redirect' => null,
                'message' => $message
            ];
        } else {
            $return = [
                'success' => true,
                'redirect' => $result['url'],
                'message' => 'Redirecting to... ' . $result['url']
            ];

            if (Settings::PAYMENT_TYPE_IFRAME === $this->config->get('payment_verifone_hosted_cart_payment_type')) {
                $scheme = parse_url($result['url'], PHP_URL_SCHEME);
                $host = parse_url($result['url'], PHP_URL_HOST);
                $return['iframe_src'] = sprintf('%s://%s/v2/loader.js?checkoutId=%s', $scheme, $host, $result['id']);
            }
            if ($this->config->get('payment_verifone_hosted_pending_status_id') != Settings::MISSING_ORDER) {
                $verifoneHelpers = new Verifone_Helpers($this->registry);
                $verifoneHelpers->addOrderHistory(
                    $this->model_checkout_order,
                    $order_id,
                    (int)$this->config->get('payment_verifone_hosted_pending_status_id'),
                    'Waiting for client to finalize the payment'
                );
            }
        }

        return json_encode($return);
    }

    /**
     * @return string
     */
    private function getLocale()
    {
        $locale = $this->language->data['code'];

        if (in_array($locale, ['nn', 'nb', 'no-nn', 'no-nb']))
            $locale = 'no';
        elseif ('nl' === $locale)
            $locale = 'nl-be';

        return $locale;
    }

    /**
     * @return string
     */
    private function getDeviceType()
    {
        $device_type = 'S';
        $checker = new Mobile_Detect();
        if ($checker->isMobile() || $checker->isTablet()) {
            $device_type = 'P';
        }

        return $device_type;
    }

    public function buildOrder($order_info, $customer_id, $total)
    {

        $params = [
            'interaction_type' => $this->settings->getPaymentType(),
            'customer' => (string)$customer_id,
            'amount' => $this->calculateAmount($total, $order_info['currency_code']),
            'currency_code' => strtoupper($order_info['currency_code']),
            'entity_id' => $this->settings->getEntityId(),
            'merchant_reference' => (string)$order_info['order_id'],
            'return_url' => $this->url->link('extension/payment/verifone_hosted/callback', '', true),
            'configurations' => [],
            'i18n' => [
                'default_language' => $this->getLocale(),
            ]
        ];

        $transactionMode = $this->getDeviceType();

        $this->addThemeId($params)
            ->addCard($params, $transactionMode, $total)
            ->addFraudCheck($params)
            ->addGiftCard($params)
            ->addApplePay($params, $transactionMode)
            ->addGooglePay($params, $transactionMode)
            ->addVipps($params, $transactionMode)
            ->addMobilePay($params, $transactionMode)
            ->addKlarna($params)
            ->addSwish($params, $transactionMode)
            ->addGpp2($params, $transactionMode)
            ->addPlcc($params)
            ->addPayPal($params);
        return $params;
    }

    protected function addThemeId(&$params)
    {
        if (!empty($this->settings->getThemeId())) {
            $params['theme_id'] = $this->settings->getThemeId();
        }
        return $this;
    }

    protected function addCard(&$params, $transactionMode, $total)
    {
        if ($this->settings->isCardEnabled()) {
            $params['configurations']['card'] = [
                'capture_now' => $this->settings->getCapture(),
                'payment_contract_id' => $this->settings->getPaymentContractId(),
                'authorization_type' => $this->settings->getTransactionType()
            ];

            if ($this->settings->is3dsEnabled()) {
                $params['configurations']['card']['threed_secure'] = [
                    'threeds_contract_id' => $this->settings->get3dsContractId(),
                    'enabled' => true,
                    'transaction_mode' => $transactionMode,
                ];
            }
            $this->absInstallments($params, $total);
        }
        return $this;
    }

    protected function absInstallments(&$params, $total)
    {
        if ($this->settings->getIsAbsEnabled() && $this->settings->getAbsInstallments($total)) {
            $params['configurations']['card']['credit_term'] = 'STANDARD';
            $nrOfInstallments = $this->settings->getAbsInstallments($total);
            if ($nrOfInstallments) {
                $params['configurations']['card']['credit_term'] = 'INSTALMENT_STANDARD';
                $params['configurations']['card']['instalment'] = ['max_number_of_instalments' => $nrOfInstallments];
            }
        }
    }

    protected function addFraudCheck(&$params)
    {
        if ($this->settings->isCardEnabled() && $this->settings->isFraudCheckEnabled()) {
            $params['configurations']['card']['fraud_protection_contract'] = $this->settings->getFraudCheckContractId();
        }
        return $this;
    }

    protected function addGiftCard(&$params)
    {
        if ($this->settings->getGiftCardEnabled()) {
            $params['configurations']['gift_card'] = [
                'capture_now' => $this->settings->getCapture(),
                'card' => [
                    'payment_contract_id' => $this->settings->getGiftCardContractId(),
                ]];
        }
        return $this;
    }

    protected function addPlcc(&$params)
    {
        if ($this->settings->getPlccEnabled()) {
            $params['configurations']['plcc'] = [
                'capture_now' => $this->settings->getCapture(),
                'payment_contract_id' => $this->settings->getPlccContractId(),
                'authorization_type' => $this->settings->getTransactionType()
            ];
        }
        return $this;
    }

    protected function addApplePay(&$params, $transactionMode)
    {
        if ($this->settings->isApplePayEnabled()) {
            $params['configurations']['apple_pay'] = [
                'card' => [
                    'sca_compliance_level' => $this->settings->getApplePayScaComplianceLevel(),
                    'payment_contract_id' => $this->settings->getApplePayContractId(),
                ],
                'capture_now' => $this->settings->getCapture(),
            ];

            if ('FORCE_3DS' === $this->settings->getApplePayScaComplianceLevel()) {
                $params['configurations']['apple_pay']['card']['threed_secure'] = [
                    'threeds_contract_id' => $this->settings->get3dsContractId(),
                    'transaction_mode' => $transactionMode,
                    'enabled' => true,
                ];
            }
        }

        return $this;
    }

    protected function addGooglePay(&$params, $transactionMode)
    {
        if ($this->settings->isGooglePayEnabled()) {
            $params['configurations']['google_pay'] = [
                'card' => [
                    'sca_compliance_level' => $this->settings->getGooglePayScaComplianceLevel(),
                    'payment_contract_id' => $this->settings->getGooglePayContractId(),
                ],
                'capture_now' => $this->settings->getCapture(),
            ];

            if (in_array($this->settings->getGooglePayScaComplianceLevel(), ['WALLET', 'FORCE_3DS'])) {
                $params['configurations']['google_pay']['card']['threed_secure'] = [
                    'threeds_contract_id' => $this->settings->get3dsContractId(),
                    'transaction_mode' => $transactionMode,
                ];
            }
        }

        return $this;
    }

    protected function addVipps(&$params, $transactionMode)
    {
        if ($this->settings->getIsVippsEnabled()) {
            $params['configurations']['vipps'] = [
                'capture_now' => $this->settings->getCapture(),
                'payment_contract_id' => $this->settings->getVippsContractId(),
                'card' => [
                    'sca_compliance_level' => $this->settings->getVippsScaComplianceLevel(),
                    'authorization_type' => $this->settings->getTransactionType()
                ]
            ];

            if (in_array($this->settings->getVippsScaComplianceLevel(), ['WALLET', 'FORCE_3DS'])) {
                $params['configurations']['vipps']['card']['threed_secure'] = [
                    'enabled' => true,
                    'threeds_contract_id' => $this->settings->getVipps3DSContractId(),
                    'transaction_mode' => $transactionMode,
                ];
            }
        }

        return $this;
    }

    protected function addGpp2(&$params, $transactionMode)
    {
        if ($this->settings->getIsGpp2Enabled()) {
            $params['configurations']['gpp2'] = [
                'payment_contract_id' => $this->settings->getGooglePayContractId(),
                'shopper_interaction' => $transactionMode === 'S' ? 'ECOMMERCE' : 'MCOMMERCE'
            ];
        }
        return $this;
    }

    protected function addSwish(&$params, $transactionMode)
    {
        if ($this->settings->getIsSwishEnabled()) {
            $params['configurations']['swish'] = [
                'shopper_interaction' => $transactionMode === 'S' ? 'ECOMMERCE' : 'MCOMMERCE'
            ];
        }
        return $this;
    }

    protected function addKlarna(&$params)
    {
        if ($this->settings->getIsKlarnaEnabled()) {
            $products = (new ModelCheckoutOrder($this->registry))->getOrderProducts($params['merchant_reference']);
            $params['configurations']['klarna'] = [
                'capture_now' => $this->settings->getCapture(),
            ];
            $currency = $params['currency_code'];
            foreach ($products as $product) {
                $totalAmount = $this->calculateAmount($product['total'], $currency);
                $taxAmount = ($this->calculateAmount($product['tax'], $currency)) * $product['quantity'];
                $unitPrice = $this->calculateAmount($product['price'], $currency);
                $rate = ($product['tax'] * 100) / $product['price'];
                $grandTotal = (int)$totalAmount + (int)$taxAmount;

                $params['line_items'][] = [
                    'name' => $product['name'],
                    'description' => substr($product['model'], 0, 126),
                    'quantity' => $product['quantity'],
                    'unit_price' => $unitPrice,
                    'tax_rate' => number_format($rate, 2),
                    'total_tax_amount' => $taxAmount,
                    'total_amount' => $grandTotal,
                ];
            }

            if (isset($this->session->data['shipping_method']['cost'])) {
                $shipping = $this->session->data['shipping_method'];
                $params['line_items'][] = [
                    'name' => $shipping['title'],
                    'description' => $shipping['title'],
                    'quantity' => 1,
                    'unit_price' => $this->calculateAmount($shipping['cost'], $currency),
                    'tax_rate' => 0,
                    'total_tax_amount' => 0,
                    'total_amount' => $this->calculateAmount($shipping['cost'], $currency),
                ];
            }
        }
        return $this;
    }

    protected function addMobilePay(&$params, $transactionMode)
    {
        if ($this->settings->getIsMobilePayEnabled()) {
            $params['configurations']['mobile_pay'] = [
                'capture_now' => $this->settings->getCapture(),
                'payment_contract_id' => $this->settings->getMobilePayContractId(),
                'card' => [
                    'sca_compliance_level' => $this->settings->getMobilePayScaComplianceLevel(),
                    'authorization_type' => $this->settings->getTransactionType()
                ]
            ];

            if (in_array($this->settings->getMobilePayScaComplianceLevel(), ['WALLET', 'FORCE_3DS'])) {
                $params['configurations']['mobile_pay']['card']['threed_secure'] = [
                    'enabled' => true,
                    'threeds_contract_id' => $this->settings->getMobilePay3DSContractId(),
                    'transaction_mode' => $transactionMode,
                ];
            }
        }

        return $this;
    }

    /**
     * @param $params
     *
     * @return void
     */
    private function addPayPal(&$params)
    {
        if ($this->settings->isPayPalEnabled()) {
            $params['configurations']['paypal'] = [
                'capture_now' => $this->settings->getCapture(),
                'payment_contract_id' => $this->settings->getPayPalContractId(),
            ];
        }
    }

    /**
     * @param $params
     * @param $order
     * @return void
     */
    private function getBillingAddress(&$params, $order)
    {

        if (!empty($order['payment_address_1'])) {
            $params['billing']['address_1'] = substr($this->replaceIllegalCharacters($order['payment_address_1']), 0, 40);
        }
        if (!empty($order['payment_address_2'])) {
            $params['billing']['address_2'] = substr($this->replaceIllegalCharacters($order['payment_address_2']), 0, 40);
        }
        if (!empty($order['payment_city'])) {
            $params['billing']['city'] = substr($this->replaceIllegalCharacters($order['payment_city']), 0, 50);
        }
        if (!empty($order['payment_iso_code_2'])) {
            $params['billing']['country_code'] = strtoupper($this->replaceIllegalCharacters($order['payment_iso_code_2']));
        }
        if (!empty($order['payment_firstname'])) {
            $params['billing']['first_name'] = substr($this->replaceIllegalCharacters($order['payment_firstname']), 0, 50);
        }
        if (!empty($order['payment_lastname'])) {
            $params['billing']['last_name'] = substr($this->replaceIllegalCharacters($order['payment_lastname']), 0, 50);
        }
        if (!empty($order['payment_postcode'])) {
            $params['billing']['postal_code'] = substr($this->replaceIllegalCharacters($order['payment_postcode'], ''), 0, 10);
        }
        if (!empty($order['payment_zone_code'])) {
            $params['billing']['state'] = substr($this->replaceIllegalCharacters($order['payment_zone_code']), 0, 50);
        }
        if (!empty($order['telephone'])) {
            $params['billing']['phone'] = $this->formatPhoneNumber($this->replaceIllegalCharacters($order['telephone'], ''));
        }

    }

    /**
     * @param $params
     * @param $order
     * @return void
     */
    private function getShippingAddress(&$params, $order)
    {

        if (empty($order['shipping_address_1'])) {
            $params['shipping'] = $params['billing'];
        } else {
            $params['shipping']['address_1'] = substr($this->replaceIllegalCharacters($order['shipping_address_1']), 0, 40);
            $params['shipping']['phone'] = $this->formatPhoneNumber($this->replaceIllegalCharacters($order['telephone'], ''));
            if (!empty($order['shipping_address_2'])) {
                $params['shipping']['address_2'] = substr($this->replaceIllegalCharacters($order['shipping_address_2']), 0, 40);
            }
            if (!empty($order['shipping_city'])) {
                $params['shipping']['city'] = substr($this->replaceIllegalCharacters($order['shipping_city']), 0, 50);
            }
            if (!empty($order['shipping_iso_code_2'])) {
                $params['shipping']['country_code'] = strtoupper($this->replaceIllegalCharacters($order['shipping_iso_code_2']));
            }
            if (!empty($order['shipping_firstname'])) {
                $params['shipping']['first_name'] = substr($this->replaceIllegalCharacters($order['shipping_firstname']), 0, 50);
            }
            if (!empty($order['shipping_lastname'])) {
                $params['shipping']['last_name'] = substr($this->replaceIllegalCharacters($order['shipping_lastname']), 0, 50);
            }
            if (!empty($order['shipping_postcode'])) {
                $params['shipping']['postal_code'] = substr($this->replaceIllegalCharacters($order['shipping_postcode'], ''), 0, 10);
            }
            if (!empty($order['shipping_zone_code'])) {
                $params['shipping']['state'] = substr($this->replaceIllegalCharacters($order['shipping_zone_code']), 0, 50);
            }
        }

    }


    /**
     * @param $data
     * @return false|string
     */
    private function toJson($data)
    {
        return json_encode($data, JSON_UNESCAPED_UNICODE);
    }

    /**
     * @param $nr
     * @return string
     */
    private function formatPhoneNumber($nr)
    {
        return (string)preg_replace('/\D/', '', $nr);
    }

    private function buildCustomer($order)
    {

        $params = [
            'entity_id' => $this->settings->getEntityId(),
        ];
        $this->getBillingAddress($params, $order);
        $this->getShippingAddress($params, $order);

        if (!empty($order['email'])) {
            $params['email_address'] = substr($this->replaceIllegalCharacters($order['email'], ''), 0, 255);
        }

        if (!empty($order['telephone'])) {
            $params['phone_number'] = $this->formatPhoneNumber($this->replaceIllegalCharacters($order['telephone'], ''));
        }

        if ($order['payment_company'] !== '') {
            $params['company_name'] = substr($this->replaceIllegalCharacters($order['payment_company']), 0, 100);
        }

        return $params;
    }


    /**
     * @param $address
     * @param $total
     * @return array
     */
    public function getMethod($address, $total)
    {
        $this->load->language('extension/payment/verifone_hosted');
        $query = $this->db->query("SELECT * FROM " . DB_PREFIX . "zone_to_geo_zone WHERE geo_zone_id = '" . (int)$this->config->get('payment_verifone_hosted_geo_zone_id') . "' AND country_id = '" . (int)$address['country_id'] . "' AND (zone_id = '" . (int)$address['zone_id'] . "' OR zone_id = '0')");
        if ($this->config->get('payment_verifone_hosted_total') > 0 && $this->config->get('payment_verifone_hosted_total') > $total) {
            $status = false;
        } elseif (!$this->config->get('payment_verifone_hosted_geo_zone_id')) {
            $status = true;
        } elseif ($query->num_rows) {
            $status = true;
        } else {
            $status = false;
        }

        $method_data = [];

        if ($status) {
            $method_data = [
                'code' => 'verifone_hosted',
                'title' => $this->language->get('text_title'),
                'terms' => '',
                'sort_order' => $this->config->get('payment_verifone_hosted_sort_order') ?? 1
            ];
        }
        return $method_data;
    }

    /**
     * @param string $subject
     * @param string $replace
     * @return string
     */
    public function replaceIllegalCharacters(string $subject, string $replace = ',')
    {
        $charset = ['/', ':', ';', '\'', '-'];
        return str_replace($charset, $replace, $subject);
    }

    public function calculateAmount($amount, string $currency, $multiply = true)
    {

        $_0decimals = ['XPF', 'XOF', 'XAF', 'VUV', 'VND', 'UYI', 'UGX', 'RWF', 'PYG', 'KRW', 'KMF', 'JPY', 'ISK', 'GNF', 'DJF', 'CLP', 'BIF'];
        $_3decimals = ['TND', 'OMR', 'LYD', 'KWD', 'JOD', 'IQD', 'BHD'];
        $_4decimals = ['UYW', 'CLF'];
        $stringAmount = (string)$amount;

        switch (strtoupper($currency)) {
            case in_array($currency, $_0decimals):
                $multiplier = 1;
                break;
            case in_array($currency, $_3decimals):
                $multiplier = 1000;
                break;
            case in_array($currency, $_4decimals):
                $multiplier = 10000;
                break;
            default: // for two decimals for the rest of currencies (vast majority)
                $multiplier = 100;
                break;
        }
        $newAmount = ($multiply) ? $stringAmount * $multiplier : $stringAmount / $multiplier;

        return (string)$newAmount;
    }

}
